Package: bkmrhat
Title: Parallel Chain Tools for Bayesian Kernel Machine Regression
Version: 1.1.7
Date: 2025-11-19
Authors@R: 
    person("Alexander", "Keil", email="alex.keil@nih.gov", role = c("aut", "cre"))
Description: Bayesian kernel machine regression (from the 'bkmr' package)
    is a Bayesian semi-parametric generalized linear model approach under
    identity and probit links. There are a number of functions in this
    package that extend Bayesian kernel machine regression fits to allow
    multiple-chain inference and diagnostics, which leverage functions
    from the 'future', 'rstan', and 'coda' packages.  Reference: Bobb, J.
    F., Henn, B. C., Valeri, L., & Coull, B. A. (2018). Statistical
    software for analyzing the health effects of multiple concurrent
    exposures via Bayesian kernel machine regression. ;
    <doi:10.1186/s12940-018-0413-y>.
License: GPL (>= 3)
URL: https://github.com/alexpkeil1/bkmrhat/
BugReports: https://github.com/alexpkeil1/bkmrhat/issues
Depends: coda, R (>= 3.5.0)
Imports: bkmr, data.table, future, rstan
Suggests: knitr, markdown
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-11-19 15:57:20 UTC; keilap
Author: Alexander Keil [aut, cre]
Maintainer: Alexander Keil <alex.keil@nih.gov>
Repository: CRAN
Date/Publication: 2025-11-30 09:20:14 UTC
Built: R 4.6.0; ; 2026-04-25 13:36:11 UTC; unix
