Package: bkmrhat
Title: Parallel Chain Tools for Bayesian Kernel Machine Regression
Version: 1.1.2
Date: 2022-03-07
Authors@R: 
    person("Alexander", "Keil", , "akeil@unc.edu", role = c("aut", "cre"))
Author: Alexander Keil [aut, cre]
Maintainer: Alexander Keil <akeil@unc.edu>
Description: Bayesian kernel machine regression (from the 'bkmr' package)
    is a Bayesian semi-parametric generalized linear model approach under
    identity and probit links. There are a number of functions in this
    package that extend Bayesian kernel machine regression fits to allow
    multiple-chain inference and diagnostics, which leverage functions
    from the 'future', 'rstan', and 'coda' packages.  Reference: Bobb, J.
    F., Henn, B. C., Valeri, L., & Coull, B. A. (2018). Statistical
    software for analyzing the health effects of multiple concurrent
    exposures via Bayesian kernel machine regression. ;
    <doi:10.1186/s12940-018-0413-y>.
License: GPL (>= 3)
Depends: coda, R (>= 3.5.0)
Imports: bkmr, data.table, future, rstan
Suggests: knitr, markdown
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.1.2
NeedsCompilation: no
Packaged: 2022-03-07 13:57:23 UTC; akeil
Repository: CRAN
Date/Publication: 2022-03-07 14:20:08 UTC
