# Generated by roxygen2: do not edit by hand

S3method(as.mcmc,bkmrfit)
S3method(as.mcmc,list.bkmrfit.list)
S3method(predict,bkmrfit)
export(ExtractPIPs_parallel)
export(OverallRiskSummaries_parallel)
export(PredictorResponseBivar_parallel)
export(PredictorResponseUnivar_parallel)
export(SamplePred_parallel)
export(SingVarRiskSummaries_parallel)
export(comb_bkmrfits)
export(comb_bkmrfits_lowmem)
export(kmbayes_combine)
export(kmbayes_combine_lowmem)
export(kmbayes_continue)
export(kmbayes_diag)
export(kmbayes_diagnose)
export(kmbayes_parallel)
export(kmbayes_parallel_continue)
import(bkmr)
import(future)
importFrom(bkmr,ExtractPIPs)
importFrom(bkmr,OverallRiskSummaries)
importFrom(bkmr,PredictorResponseBivar)
importFrom(bkmr,PredictorResponseUnivar)
importFrom(bkmr,SamplePred)
importFrom(bkmr,SingVarRiskSummaries)
importFrom(coda,as.mcmc)
importFrom(coda,as.mcmc.list)
importFrom(coda,mcmc)
importFrom(coda,mcmc.list)
importFrom(data.table,as.data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(future,value)
importFrom(rstan,Rhat)
importFrom(rstan,ess_bulk)
importFrom(rstan,ess_tail)
importFrom(stats,predict)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
